/*- 
 * Copyright (c) 1999 Chris Costello <chris@calldei.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] = "$Id$";
#endif

#include <err.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#define TEST(s1, s2, s3, s4)			\
{						\
	if (!strcmp(prg, #s1))			\
		mode = s1;			\
	else if (!strcmp(prg, #s2))		\
		mode = s2;			\
	else if (!strcmp(prg, #s3))		\
		mode = s3;			\
	else if (!strcmp(prg, #s4))		\
		mode = s4;			\
	else					\
		mode = -1;			\
}

void	usage(void);
void	noprg(void);

char *prg;

int
main(argc, argv)
	int argc;
	char **argv; 
{
	char ch, *src, *dst, *tprg;
	int mode = -1;
	int dos2bsd = 1, bsd2dos = 2, mac2bsd = 3, bsd2mac = 4;

	/*
	 * Default the input and output filenames to stdin and stdout
	 * respectively, this is what BSDI's dos2bsd/bsd2dos does.
	 */
	src = "/dev/stdin";
	dst = "/dev/stdout";

	/* prg is a pointer to the original argv[0] */
	prg = argv[0];

	/*
	 * $ dos2bsd -a -b -c
	 * dos2bsd: illegal option -- a
	 * usage: dos2bsd [input [output]]
	 * --------------------------------
	 * It looks like it uses getopt, but doesn't allow any opts (the
	 * manual page also doesn't mention any options).
	 */

	while ((ch = getopt(argc, argv, "?")) != -1)
		switch(ch) {
		case '?':
		default:
			usage();
			/* NOTREACHED */
		}

	argc -= optind;
	argv += optind;

	/*
	 * Now we see how many args we have, and set the source/destination
	 * as necessary.
	 */

	switch (argc) {
	case 2:
		if (freopen(dst = argv[1], "w", stdout) == NULL)
			err(1, "%s", argv[1]);
		/* FALLTHROUGH */
	case 1:
		if (freopen(dst = argv[0], "r", stdin) == NULL)
			err(1, "%s", argv[0]);
		/* FALLTHROUGH */
	case 0:
		break;
	default:
		usage();
		/* NOTREACHED */
	}

	/* Now determine the program to use */
	tprg = strchr(prg, '/');

	if (tprg != NULL)
		if ((tprg += 1) != NULL)
			prg = tprg;
		
	TEST(dos2bsd, bsd2dos, mac2bsd, bsd2mac);
	if (mode == -1)
		usage();
		

	switch (mode) {
	case 1:
		while ((ch = getchar()) != EOF) {
			if (ch != '\r')
				putchar(ch);
		}
		break;
	case 2:
		while ((ch = getchar()) != EOF) {
			if (ch == '\n')
				putchar('\r');
			putchar(ch);
		}
		break;
	case 3: 
		while ((ch = getchar()) != EOF) {
			if (ch == '\r')
				putchar('\n');
			else
				putchar(ch);
		}
		break;
	case 4:
		while ((ch = getchar()) != EOF) {
			if (ch == '\n')
				putchar('\r');
			else
				putchar(ch);
		}
		break;
	default:
		noprg();
		/* NOTREACHED */
	}

	if (ferror(stdin))
		err(1, "%s", src);
	if (ferror(stdout))
		err(1, "%s", dst);

	fclose(stdin);
	fclose(stdout);

	exit(0);
}

void
noprg()
{
	errx(1, "This can only be run as dos2bsd, bsd2dos, mac2bsd, or bsd2mac");
	/* NOTREACHED */
}

void
usage()
{
	fprintf(stderr, "usage: %s [input [output]]\n", prg);
	exit(1);
}

